
# q-expansion of newform 1210.2.b.f, downloaded from the LMFDB on 02 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1210
weight = 2
poly_data = [1, 0, 3, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[1, 0, 1, 0], 1], [[0, 2, 0, 1], 1]]

hecke_ring_character_values = [[727, [-1, 0, 0, 0]], [1091, [1, 0, 0, 0]]]
aps_data = [[0, 0, 0, -1], [0, 2, 0, 0], [-2, 0, 0, -1], [0, 1, 0, 0], [0, 0, 0, 0], [0, 1, 0, -3], [0, 4, 0, 4], [-3, 0, -1, 0], [0, -1, 0, -4], [-2, 0, -4, 0], [2, 0, 0, 0], [0, 1, 0, 5], [0, 0, 1, 0], [0, 4, 0, 6], [0, 5, 0, 7], [0, 5, 0, 4], [9, 0, 3, 0], [-2, 0, -8, 0], [0, 2, 0, 8], [2, 0, 0, 0], [0, 4, 0, -4], [0, 0, 10, 0], [0, 10, 0, 4], [-12, 0, 1, 0], [0, 2, 0, 8], [-16, 0, -6, 0], [0, -3, 0, -5], [0, 12, 0, 8], [8, 0, -4, 0], [0, -12, 0, -2], [0, 3, 0, 1], [-8, 0, 0, 0], [0, -12, 0, -14], [4, 0, -7, 0], [-18, 0, -6, 0], [10, 0, 6, 0], [0, -3, 0, 8], [0, -12, 0, -2], [0, -7, 0, -9], [0, -3, 0, 0], [0, 0, 15, 0], [10, 0, 16, 0], [-14, 0, 8, 0], [0, -2, 0, -12], [0, -23, 0, -12], [8, 0, -4, 0], [24, 0, 4, 0], [0, -5, 0, -16], [0, 0, 0, 12], [6, 0, 2, 0], [0, 18, 0, 8], [-6, 0, -2, 0], [-8, 0, 5, 0], [0, 0, 11, 0], [0, 8, 0, 16], [0, 3, 0, -7], [-10, 0, -20, 0], [6, 0, -12, 0], [0, -13, 0, 8], [-14, 0, 8, 0], [0, 10, 0, -6], [0, 3, 0, 28], [0, -2, 0, -14], [2, 0, -10, 0], [0, 4, 0, -14], [0, 1, 0, -15], [-3, 0, -5, 0], [0, 10, 0, -8], [0, 14, 0, -6], [6, 0, 2, 0], [0, 0, 0, -16], [20, 0, 20, 0], [0, 16, 0, 20], [0, 9, 0, 1], [-24, 0, -13, 0], [0, 23, 0, 8], [-14, 0, -18, 0], [0, 7, 0, 8], [-3, 0, 5, 0], [-13, 0, -21, 0], [-23, 0, -21, 0], [2, 0, 0, 0], [22, 0, 10, 0], [0, 12, 0, 0], [0, 0, 10, 0], [0, 10, 0, -6], [19, 0, 3, 0], [0, 10, 0, 2], [-6, 0, -6, 0], [0, -7, 0, 3], [0, -14, 0, -10], [26, 0, 2, 0], [0, 8, 0, 36], [17, 0, 15, 0], [-4, 0, 7, 0], [0, 1, 0, -13], [8, 0, 6, 0], [-28, 0, -5, 0], [0, -18, 0, 10], [22, 0, -10, 0], [0, 16, 0, 10], [0, -9, 0, -5], [0, -22, 0, -32], [-4, 0, -3, 0], [28, 0, 27, 0], [0, -4, 0, 20], [0, -10, 0, -18], [0, -16, 0, 16], [0, 0, 10, 0], [-3, 0, 5, 0], [0, 4, 0, -36], [0, 0, 0, -6], [0, 8, 0, -14], [13, 0, -9, 0], [16, 0, 18, 0], [19, 0, 19, 0], [0, 24, 0, 6], [0, 32, 0, 8], [0, -23, 0, -15], [12, 0, -1, 0], [4, 0, -16, 0], [0, -10, 0, 4], [0, 4, 0, 14], [0, -22, 0, -2], [20, 0, 31, 0], [-8, 0, -30, 0], [2, 0, 4, 0], [-6, 0, -22, 0], [0, -19, 0, -20], [0, -12, 0, -2], [-8, 0, -1, 0], [0, 7, 0, -20], [-12, 0, -18, 0], [0, -15, 0, 17], [2, 0, -20, 0], [29, 0, 13, 0], [0, 25, 0, 24], [0, -14, 0, -10], [0, 1, 0, 5], [-3, 0, 9, 0], [-44, 0, 3, 0], [-6, 0, 14, 0], [0, -25, 0, 9], [0, 24, 0, -6], [28, 0, -4, 0], [6, 0, 2, 0], [0, -15, 0, 4], [0, 10, 0, 2], [-8, 0, -1, 0], [0, -13, 0, -15], [0, -37, 0, -29], [0, 0, 11, 0], [0, 6, 0, -18], [0, 31, 0, 20], [0, 38, 0, 6], [-18, 0, -10, 0], [-10, 0, 0, 0], [-4, 0, 17, 0], [0, 28, 0, 36], [10, 0, 16, 0], [0, 0, 0, -28], [0, -2, 0, 8], [0, 9, 0, 39], [-16, 0, -31, 0], [0, 24, 0, -6], [0, 23, 0, 8], [42, 0, -10, 0], [0, -8, 0, -22], [-35, 0, 5, 0], [0, -4, 0, 22], [19, 0, -7, 0], [0, 0, -4, 0], [38, 0, 2, 0], [0, -24, 0, -48], [-4, 0, -28, 0], [31, 0, 7, 0], [-35, 0, 11, 0], [22, 0, 0, 0], [0, 27, 0, 40], [24, 0, -12, 0], [0, 4, 0, 4], [5, 0, -9, 0], [0, 8, 0, -22], [0, 2, 0, -12], [0, 11, 0, -23], [-26, 0, -12, 0], [0, -20, 0, -18], [0, 42, 0, 20], [10, 0, 20, 0], [22, 0, 0, 0], [0, -6, 0, -14], [0, 16, 0, 22], [-13, 0, -15, 0], [-18, 0, 20, 0], [0, 18, 0, -14], [0, -26, 0, 16], [13, 0, 17, 0], [0, -44, 0, -38], [0, 6, 0, -20], [0, -13, 0, 0], [30, 0, 0, 0], [4, 0, 34, 0], [0, -41, 0, -1], [14, 0, -12, 0], [28, 0, 16, 0], [0, 27, 0, -17], [-18, 0, -6, 0], [0, 4, 0, -44], [36, 0, 37, 0], [32, 0, 20, 0], [0, 0, 0, 32], [24, 0, -6, 0], [0, -5, 0, -36], [0, -34, 0, -10], [-24, 0, -18, 0], [-37, 0, 7, 0], [0, 24, 0, 4], [-43, 0, 5, 0], [0, -12, 0, -44], [0, 12, 0, 50], [-40, 0, -44, 0], [-2, 0, 36, 0], [-32, 0, -49, 0], [0, -41, 0, -24], [0, -6, 0, -26], [16, 0, 12, 0], [0, 8, 0, 58], [-28, 0, -56, 0], [0, 43, 0, 21], [36, 0, 53, 0], [0, 37, 0, 25], [-12, 0, 11, 0], [32, 0, 20, 0], [-20, 0, -29, 0], [0, -14, 0, -58], [0, 16, 0, 40], [27, 0, -1, 0], [0, -37, 0, -17], [24, 0, 48, 0], [-46, 0, -26, 0], [0, -14, 0, 32], [-52, 0, -13, 0], [0, 3, 0, 8], [22, 0, 4, 0], [0, -34, 0, 2], [34, 0, 28, 0], [0, -3, 0, 23], [56, 0, 28, 0], [32, 0, 24, 0], [0, 57, 0, 20], [0, -13, 0, -52], [27, 0, 35, 0], [0, -33, 0, 13], [-40, 0, -5, 0], [0, -37, 0, -17], [-37, 0, -39, 0], [-12, 0, -18, 0], [0, 20, 0, -8], [0, -15, 0, -3], [0, -12, 0, 6], [0, -40, 0, 4], [0, -12, 0, 36], [-10, 0, 0, 0], [0, -35, 0, 4], [0, -2, 0, 26], [-17, 0, -19, 0], [32, 0, 44, 0], [39, 0, -1, 0], [0, 18, 0, 18], [0, 9, 0, 41], [22, 0, 50, 0], [0, 20, 0, 12], [0, 48, 0, 8], [-64, 0, -18, 0], [0, 16, 0, 20], [0, 29, 0, -29], [0, 32, 0, 28], [46, 0, -18, 0], [-11, 0, -51, 0], [-40, 0, -24, 0], [0, -31, 0, 16], [16, 0, 38, 0], [0, 23, 0, 41], [-38, 0, -50, 0], [0, 8, 0, 36], [36, 0, 58, 0], [0, 4, 0, -14], [0, 3, 0, -29], [-6, 0, 38, 0], [4, 0, -37, 0], [10, 0, 6, 0], [0, 4, 0, 44], [0, -2, 0, 48], [4, 0, -31, 0], [0, 49, 0, 56], [30, 0, 50, 0], [10, 0, 26, 0], [0, -1, 0, 16], [13, 0, 51, 0], [0, -30, 0, -28], [0, -20, 0, 34], [0, -27, 0, -64], [26, 0, 52, 0]]
